import sys 
sys.path.append(".")
from forum.handler import BaseHandler
from forum.wtforms import UserForm
from forum.models import UserModel
from uuid import uuid4
from forum.utils.email_util import send_mail
from config import email,secret,jwt_exp
from random import choice
from datetime import datetime
# https://jwt.io/#libraries-io
import jwt
# from forum.decorators import login_required_async
import os
from config import settings
from forum.decorators import login_required_async


#登录功能
class LoginUserHandler(BaseHandler.BaseHandler):
    async def post(self):
        rs_data = {}
        user_form = UserForm(self.request.arguments)
        # user_form.nick_name.data= '1111'
        del user_form['nick_name']
        if user_form.validate():
            email = user_form.email.data
            password = user_form.password.data
            try:
                user = await self.obj.get(UserModel,email = email,password = password)
                if user:
                    rs_data['msg']='登录成功'
                    rs_data['code']=200
                    payload = {
                        'email':email,
                        'nick_name':user.nick_name,
                        'exp':datetime.utcnow()
                    }
                    token = jwt.encode(payload,secret,algorithm = 'HS256') 
                    rs_data['token'] = token
            except Exception as e:
                rs_data['msg']='账号或者密码错误'
                rs_data['code']=401
                return self.finish(rs_data)
        else:
            for field in user_form.errors:
                rs_data[field] = user_form.errors[field][0]
            rs_data['code']=500
        return self.finish(rs_data)

#获取用户功能
class GetUserHandler(BaseHandler.BaseHandler):
    @login_required_async
    async def get(self):
        rs_data = {}
        uid = self._user_id
        # token = self.request.headers.get('token',None)
        # try:
        #     if token:
        #         payload = jwt.decode(token,secret,algorithms = ['HS256'],leeway = jwt_exp,options={'verify_exp':True})
        #         email = payload.get('email')
        try:
            user = await self.obj.get(UserModel,id = uid)  
            if user:       
                rs_data['msg'] = '获取用户成功'
                rs_data['code'] = 200
                rs_data['user'] = user.to_json()
        except Exception as e:
            rs_data['msg'] = '获取用户失败'
            rs_data['code'] = 500
        return self.finish(rs_data)


#注册功能
class AddUserHandler(BaseHandler.BaseHandler):
    async def post      (self):
        rs_data={}
        form = UserForm(self.request.arguments)
        tm_code = self.get_body_argument('code')
        if form.validate():
            email = form.email.data
            try:
                existed_user= await self.obj.get(UserModel,email = email)
                if existed_user:
                    rs_data['msg']='用户名已存在'
                    rs_data['code']=500
            except Exception as e:  
                code = self.redis.get(email)#获取
                if code and code.decode() == tm_code: #判断是否有验证码
                    form.id.data=uuid4()
                    #有值就可以注册
                    await self.obj.create(UserModel,**form.data)
                    rs_data['msg']='注册成功'
                    rs_data['code']=200
                else:
                    rs_data['msg']='验证码错误'
                    rs_data['code']=500
        else:
            rs_data['code'] = 500
            rs_data['msg'] = '注册失败'
            for field in form.errors:
                rs_data[field] = form.errors[field][0]
        return self.finish(rs_data)

#发送验证码邮件
class SendEmail(BaseHandler.BaseHandler):
    def generate_code(self):
        base = '0123456789'
        code = []
        #随机在0-9拿四个数字
        for i in range(4):
            code.append(choice(base))
        return ''.join(code)#转为字符串

    async def post(self):
        user_mail = self.get_body_argument('email')
        code = self.generate_code()
        # await send_mail(email.get('uname'),email.get('password'),user_mail,'<尚学堂-论坛>用户注册',"您现在注册的验证码为:"+self.generate_code())
        msg = f'您好，您现在使用{user_mail}注册<尚学堂-论坛>网站，您的验证码是：{ code }。如非本人操作请忽略。'
        send_mail(
            email.get('uname'),
            email.get('password'),
            user_mail,
            '<尚学堂-论坛>用户注册',
            msg)
        self.redis.set(user_mail,code)
        print(self.redis.get(user_mail))

#更新用户信息
class UpdateUserHandler(BaseHandler.BaseHandler):
    @login_required_async
    async def post(self):
        user_form = UserForm(self.request.arguments)
        if user_form.validate():
            uid = user_form.id.data
            rs = await self.obj.execute(UserModel.update(**user_form.data).where(UserModel.id == uid))
            if rs > 0:
                return self.finish({'msg':'更新用户成功','code':200})
            else:
                return self.finish({'msg':'更新用户失败','code':500})
        else:
            pass
